@@ -62,6 +62,8 @@
 #include "gdm-daemon-config.h"
 
 extern char **environ;
+extern pid_t gdm_main_pid;
+extern pid_t extra_process;
 
 #ifdef ENABLE_IPV6
 
@@ -295,12 +297,13 @@ gdm_get_free_display (int start, uid_t server_uid)
 	 * go that far
          */
 	for (i = start; i < 3000; i++) {
+		FILE *fp;
 		GSList *li;
+		GSList *displays;
 		struct stat s;
 		char buf[256];
-		FILE *fp;
 		int r;
-                GSList *displays;
+		gboolean try_ipv4 = TRUE;
 
                 displays = gdm_daemon_config_get_display_list ();
 
@@ -328,9 +331,20 @@ gdm_get_free_display (int start, uid_t server_uid)
 			VE_IGNORE_EINTR (connect (sock,
                                       (struct sockaddr *)&serv6_addr,
                                       sizeof (serv6_addr)));
+
+			/*
+			 * If IPv6 returns the network is unreachable,
+			 * then try fallbacking to IPv4.  In all other
+			 * cases, do not fallback.  This problem can
+			 * happen if IPv6 is enabled, but the
+			 * administrator has disabled it.
+			 */
+			if (errno != ENETUNREACH)
+				try_ipv4 = FALSE;
 		}
-		else
 #endif
+
+		if (try_ipv4)
 		{
 			sock = socket (AF_INET, SOCK_STREAM, 0);
 
@@ -2168,4 +2182,52 @@ gdm_read_default (gchar *key)
 #endif
 }
 
+/**
+ * gdm_fail:
+ * @format: printf style format string
+ * @...: Optional arguments
+ *
+ * Logs fatal error condition and aborts master daemon.  Also sleeps
+ * for 30 seconds to avoid looping if gdm is started by init.
+ */
+void
+gdm_fail (const gchar *format, ...)
+{
+    va_list args;
+    char *s;
+
+    va_start (args, format);
+    s = g_strdup_vprintf (format, args);
+    va_end (args);
+
+    /* Log to both syslog and stderr */
+    gdm_error (s);
+    if (getpid () == gdm_main_pid) {
+	    gdm_fdprintf (2, "%s\n", s);
+    }
+
+    g_free (s);
+
+    /* If main process do final cleanup to kill all processes */
+    if (getpid () == gdm_main_pid) {
+            gdm_final_cleanup ();
+    } else if ( ! gdm_slave_final_cleanup ()) {
+            /* If we weren't even a slave do some random cleanup only */
+            /* FIXME: is this all fine? */
+            gdm_sigchld_block_push ();
+            if (extra_process > 1 && extra_process != getpid ()) {
+                    /* we sigterm extra processes, and we don't wait */
+                    kill (-(extra_process), SIGTERM);
+                    extra_process = 0;
+            }
+            gdm_sigchld_block_pop ();
+    }
+
+    /* Slow down respawning if we're started from init */
+    if (getppid () == 1)
+        sleep (30);
+
+    exit (EXIT_FAILURE);
+}
+
 /* EOF */
